@@ -31,18 +31,30 @@ use matrix_sdk::{
 };
 use serde::Deserialize;
 use std::convert::TryFrom;
+use std::sync::Arc;
 use tokio::fs;
+use tokio::sync::RwLock;
 use tracing::{debug, error, info};
 use tracing_subscriber::EnvFilter;
 
+mod collections;
+
 /// Reasons from an IRC appservice to ignore
-const IRC_IGNORE_REASONS: [&str; 2] = [
+const IRC_IGNORE_REASONS: [&str; 3] = [
     // When a user uses /part on the IRC side
     "user left",
     // Leaving the IRC network via !quit
     "issued !quit command",
+    // 30 day bridge inactivity kick
+    "You have been kicked for being idle",
 ];
 
+#[derive(Clone)]
+struct State {
+    config: Config,
+    recent_kicks: Arc<RwLock<collections::ExpiringSet>>,
+}
+
 #[derive(Clone, Deserialize)]
 struct Config {
     username: String,
@@ -93,7 +105,7 @@ async fn handle_event(
     ev: SyncStateEvent<MemberEventContent>,
     client: Client,
     room: Room,
-    config: Config,
+    state: State,
 ) {
     let event: StateEvent<_> = ev.into_full_event(room.room_id().clone());
     debug!(event = ?event);
@@ -102,49 +114,82 @@ async fn handle_event(
         .reason
         .clone()
         .unwrap_or_else(|| "no reason provided".to_string());
-    if let MembershipChange::Kicked = event.membership_change() {
-        // TODO: This should be configurable for other bridges, e.g. OFTC
-        if event.sender == user_id!("@appservice:libera.chat")
-            && IRC_IGNORE_REASONS.contains(&reason.as_str())
-        {
-            debug!("Ignoring kick by IRC appservice");
-            return;
+    match event.membership_change() {
+        MembershipChange::Joined => {
+            let joined = event.sender.to_string();
+            if state.recent_kicks.read().await.contains(&joined) {
+                let content = AnyMessageEventContent::RoomMessage(
+                    MessageEventContent::text_plain(format!(
+                        "After being recently kicked, {} joined {}",
+                        &joined,
+                        room_alias(&room),
+                    )),
+                );
+                let send_room = client.get_joined_room(
+                    &RoomId::try_from(state.config.report_room.as_str())
+                        .unwrap(),
+                );
+                if let Some(room) = send_room {
+                    info!(
+                        room = %room.room_id(),
+                        content = ?content,
+                        "Sending notification"
+                    );
+                    room.send(content, None).await.unwrap();
+                } else {
+                    error!(
+                        report_room = %state.config.report_room,
+                        "Not joined to report_room"
+                    );
+                }
+            }
         }
-        // The state_key is the full user id of the kicked user
-        let kicked = event.state_key;
-        // Get a list of what other rooms they're in
-        let other_rooms = get_rooms_user_is_in(&client, &kicked).await;
-        let other_rooms_str = if other_rooms.is_empty() {
-            "".to_string()
-        } else {
-            format!(", they are still in {}", other_rooms.join(", "))
-        };
-        let content = AnyMessageEventContent::RoomMessage(
-            MessageEventContent::text_plain(format!(
-                "{} was kicked by {} from {} ({}){}",
-                &kicked,
-                event.sender,
-                room_alias(&room),
-                reason,
-                other_rooms_str
-            )),
-        );
-        let send_room = client.get_joined_room(
-            &RoomId::try_from(config.report_room.as_str()).unwrap(),
-        );
-        if let Some(room) = send_room {
-            info!(
-                room = %room.room_id(),
-                content = ?content,
-                "Sending notification"
+        MembershipChange::Kicked => {
+            // TODO: This should be configurable for other bridges, e.g. OFTC
+            if event.sender == user_id!("@appservice:libera.chat")
+                && IRC_IGNORE_REASONS.contains(&reason.as_str())
+            {
+                debug!("Ignoring kick by IRC appservice");
+                return;
+            }
+            // The state_key is the full user id of the kicked user
+            let kicked = event.state_key;
+            state.recent_kicks.write().await.add(&kicked);
+            // Get a list of what other rooms they're in
+            let other_rooms = get_rooms_user_is_in(&client, &kicked).await;
+            let other_rooms_str = if other_rooms.is_empty() {
+                "".to_string()
+            } else {
+                format!(", they are still in {}", other_rooms.join(", "))
+            };
+            let content = AnyMessageEventContent::RoomMessage(
+                MessageEventContent::text_plain(format!(
+                    "{} was kicked by {} from {} ({}){}",
+                    &kicked,
+                    event.sender,
+                    room_alias(&room),
+                    reason,
+                    other_rooms_str
+                )),
             );
-            room.send(content, None).await.unwrap();
-        } else {
-            error!(
-                report_room = %config.report_room,
-                "Not joined to report_room"
+            let send_room = client.get_joined_room(
+                &RoomId::try_from(state.config.report_room.as_str()).unwrap(),
             );
+            if let Some(room) = send_room {
+                info!(
+                    room = %room.room_id(),
+                    content = ?content,
+                    "Sending notification"
+                );
+                room.send(content, None).await.unwrap();
+            } else {
+                error!(
+                    report_room = %state.config.report_room,
+                    "Not joined to report_room"
+                );
+            }
         }
+        _ => {}
     }
 }
 
@@ -172,15 +217,19 @@ async fn main() -> Result<()> {
 
     debug!("Logged in");
     client.sync_once(SyncSettings::default()).await?;
+    let state = State {
+        config,
+        recent_kicks: Arc::new(RwLock::new(collections::ExpiringSet::new(60))),
+    };
     client
         .register_event_handler({
-            let config = config.clone();
+            let state = state.clone();
             move |ev: SyncStateEvent<MemberEventContent>,
                   client: Client,
                   room: Room| {
-                let config = config.clone();
+                let state = state.clone();
                 async move {
-                    handle_event(ev, client, room, config).await;
+                    handle_event(ev, client, room, state).await;
                 }
             }
         })
